@@ -1,6 +1,8 @@
+#include <Rmath.h>
+
 void my0F1(int *n, double *x, double *nu, double *y0, double *y)
 {
-    double r, s, t, u, v, z;
+    double m, r, s, t, u, v, z;
     int i;
 
     for(i = 0; i < *n; i++) {
@@ -12,9 +14,18 @@ void my0F1(int *n, double *x, double *nu, double *y0, double *y)
 	r = 0.;
 	s = t;
 
-	/* Iterate until numeric convergence if possible. */
+	/* Iterate until terms reach maximum. */
+	m = (- u - 1 + sqrt((u - 1) * (u - 1)  + 4 * z)) / 2 + 1;
+	while(v < m) {
+	    r = s;
+	    t *= z / (u * v);
+	    s += t;	    
+	    u++;
+	    v++;
+	}
 
-	while((s > r) && (v <= 500)) {
+	/* Iterate until numeric convergence if possible. */
+	while(s > r) {
 	    r = s;
 	    t *= z / (u * v);
 	    s += t;	    
